vss_preselect(): Use single return.
[paraslash.git] / signal.c
index 128f6bba90fd0a9fd3b1f80adf8731d685438af1..f8be2f4612002214f669fa74ce92390866b0dc90 100644 (file)
--- a/signal.c
+++ b/signal.c
@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2004-2006 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2004-2008 Andre Noll <maan@systemlinux.org>
  *
  * Licensed under the GPL v2. For licencing details see COPYING.
  */
-/** \file signal.c signal handling functions */
+/** \file signal.c Signal handling functions. */
 
 #include <signal.h>
 #include <sys/types.h>
@@ -40,15 +40,15 @@ int para_signal_init(void)
                ret = -ERRNO_TO_PARA_ERROR(errno);
                goto err_out;
        }
-       ret = mark_fd_nonblock(signal_pipe[0]);
+       ret = mark_fd_nonblocking(signal_pipe[0]);
        if (ret < 0)
                goto err_out;
-       ret = mark_fd_nonblock(signal_pipe[1]);
+       ret = mark_fd_nonblocking(signal_pipe[1]);
        if (ret < 0)
                goto err_out;
        return signal_pipe[0];
 err_out:
-       PARA_EMERG_LOG("%s\n", PARA_STRERROR(-ret));
+       PARA_EMERG_LOG("%s\n", para_strerror(-ret));
        exit(EXIT_FAILURE);
 }
 
@@ -64,7 +64,7 @@ static void generic_signal_handler(int s)
 /**
  * Reap one child.
  *
- * \para, pid In case a child died, its pid is returned here.
+ * \param pid In case a child died, its pid is returned here.
  *
  * Call waitpid() and print a log message containing the pid and the cause of
  * the child's death.
@@ -73,7 +73,7 @@ static void generic_signal_handler(int s)
  * one otherwise. If and only if the function returns one, the content of \a
  * pid is meaningful.
  *
- * \sa waitpid(2)
+ * \sa waitpid(2).
  */
 int para_reap_child(pid_t *pid)
 {
@@ -85,13 +85,13 @@ int para_reap_child(pid_t *pid)
        if (*pid < 0)
                return -ERRNO_TO_PARA_ERROR(errno);
        if (WIFEXITED(status))
-               PARA_DEBUG_LOG("child %i exited. Exit status: %i\n", *pid,
+               PARA_DEBUG_LOG("child %i exited. Exit status: %i\n", (int)*pid,
                        WEXITSTATUS(status));
        else if (WIFSIGNALED(status))
-               PARA_DEBUG_LOG("child %i was killed by signal %i\n", *pid,
+               PARA_DEBUG_LOG("child %i was killed by signal %i\n", (int)*pid,
                        WTERMSIG(status));
        else
-               PARA_WARNING_LOG("child %i terminated abormally\n", *pid);
+               PARA_WARNING_LOG("child %i terminated abormally\n", (int)*pid);
        return 1;
 }
 
@@ -110,12 +110,15 @@ void para_reap_children(void)
 }
 
 /**
- * wrapper around signal(2)
- * \param sig the number of the signal to catch
+ * Wrapper around signal(2).
+ *
+ * \param sig The number of the signal to catch.
  *
  * This installs the generic signal handler for the given signal.
+ *
  * \return This function returns 1 on success and \p -E_SIGNAL_SIG_ERR on errors.
- * \sa signal(2)
+ *
+ * \sa signal(2).
  */
 int para_install_sighandler(int sig)
 {
@@ -124,22 +127,37 @@ int para_install_sighandler(int sig)
 }
 
 /**
- * return number of next pending signal
+ * Return the number of next pending signal.
  *
  * This should be called if the fd for the signal pipe is ready for reading.
  *
* \return On success, the number of the received signal is returned. \p
* -E_SIGNAL_READ is returned if a read error occurred while reading the signal
* pipe.  If the read was interrupted by another signal the function returns 0.
  */
 int para_next_signal(void)
 {
        int s;
-       ssize_t r;
+       ssize_t r = read(signal_pipe[0], &s, sizeof(s));
 
-       if ((r = read(signal_pipe[0], &s, sizeof(s)) == sizeof(s)) > 0) {
-               PARA_DEBUG_LOG("next signal: %d\n", s);
-               return s;
+       if (!r) {
+               PARA_CRIT_LOG("read from signal pipe returned zero\n");
+               return 0;
+       }
+       if (r < 0) {
+               if (errno == EAGAIN || errno == EINTR)
+                       return 0;
+               return -ERRNO_TO_PARA_ERROR(errno);
        }
-       return r < 0 && (errno != EAGAIN)? 0 : -E_SIGNAL_READ;
+       assert(r == sizeof(s));
+       PARA_DEBUG_LOG("next signal: %d\n", s);
+       return s;
+}
+
+/**
+ * Close the write end of the signal pipe.
+ */
+void para_signal_shutdown(void)
+{
+       close(signal_pipe[1]);
 }
@@ -124,22 +127,37 @@ int para_install_sighandler(int sig)
  * This should be called if the fd for the signal pipe is ready for reading.
  *
- * \return On success, the number of the received signal is returned. \p
- * -E_SIGNAL_READ is returned if a read error occurred while reading the signal
- * pipe.  If the read was interrupted by another signal the function returns 0.
+ * \return On success, the number of the received signal is returned.  If the
+ * read returned zero or was interrupted by another signal the function returns
+ * 0.  Otherwise, a negative error value is returned.
  */
 int para_next_signal(void)